#include <stdio.h>
#include <stdlib.h>

#include "bmp.h"
#include "raw.h"



int bmp2raw(struct bmp_header bh, struct bmp_color *cmap, unsigned char *bmp,
		lcd_raw_buffer raw) {
	int w, h;
	int x, y, ofs, linesize;
	unsigned char *grey;
	linesize = ((bh.width*bh.bit_count / 8) + 3) & ~3;
	/* use min values for image size */
	w = (bh.width<120)?bh.width:120;
	h = (bh.height<64)?bh.height:64;

	grey = malloc(1<<bh.bit_count);
	for (x = 0; x < (1<<bh.bit_count); x++) {
		/* green has 50%, red and blue 25% each */
		grey[x] = (cmap[x].g>>1) + (cmap[x].r>>2) + (cmap[x].b>>2);
		/*printf("grey[%02i] = %02i (%02x %02x %02x)\n", x, grey[x],
			cmap[x].r, cmap[x].g, cmap[x].b);*/
	}

	switch (bh.bit_count) {
	case 1:
		for (y=0; y<h; y++) { for (x=0; x<w; x++) {
			ofs = (bh.height - 1 - y)*linesize + (x>>3);
			raw[y][x] = grey[(bmp[ofs]>>(7-(x&7)))&1];
		} }
		break;
	case 4:
		for (y=0; y<h; y++) { for (x=0; x<(w>>1); x++) {
			ofs = (bh.height - 1 - y)*linesize + x;
			raw[y][x*2] = grey[bmp[ofs] >> 4];
			raw[y][x*2+1] = grey[bmp[ofs] & 0x0F];
		} }
		break;
	default:
		return -1;
	}
	return 0;
}

#define swap32bits(i) (i>>24) | (i<<24) | ((i>>8) & 0x00f0) | ((i<<8) & 0x0f00)
#define swap16bits(i) (i>>8) | (i<<8)


void bh_ntoh(struct bmp_header *bh) {
	#if   __BYTE_ORDER == __BIG_ENDIAN
	bh->file_size = swap32bits(bh->file_size);
	bh->reserved = swap32bits(bh->reserved);
	bh->data_offset = swap32bits(bh->data_offset);
	bh->header_size = swap32bits(bh->header_size);
	bh->width = swap32bits(bh->width);
	bh->height = swap32bits(bh->height);
	bh->planes = swap16bits(bh->planes);
	bh->bit_count = swap16bits(bh->bit_count);
	bh->compression = swap32bits(bh->compression);
	bh->image_size = swap32bits(bh->image_size);
	bh->x_ppm = swap32bits(bh->x_ppm);
	bh->y_ppm = swap32bits(bh->y_ppm);
	bh->colors_used = swap32bits(bh->colors_used);
	bh->colors_vip = swap32bits(bh->colors_vip);
	#endif
}
void bh_hton(struct bmp_header *bh) {
	#if   __BYTE_ORDER == __BIG_ENDIAN
	bh->file_size = swap32bits(bh->file_size);
	bh->reserved = swap32bits(bh->reserved);
	bh->data_offset = swap32bits(bh->data_offset);
	bh->header_size = swap32bits(bh->header_size);
	bh->width = swap32bits(bh->width);
	bh->height = swap32bits(bh->height);
	bh->planes = swap16bits(bh->planes);
	bh->bit_count = swap16bits(bh->bit_count);
	bh->compression = swap32bits(bh->compression);
	bh->image_size = swap32bits(bh->image_size);
	bh->x_ppm = swap32bits(bh->x_ppm);
	bh->y_ppm = swap32bits(bh->y_ppm);
	bh->colors_used = swap32bits(bh->colors_used);
	bh->colors_vip = swap32bits(bh->colors_vip);
	#endif
}
